var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, {enumerable: true, configurable: true, writable: true, value}) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
import {isEmpty, isArray, getFriendlyFileSize, toArray, randomHash, humanize, isDateTime, isDate, isTime, isYear} from "boost-web-core";
import {withState, h, renderToDom, toDomElement} from "vdtree";
function getValidationResult(errorMessage) {
  return {
    message: errorMessage != null ? errorMessage : "",
    hasError: !isEmpty(errorMessage != null ? errorMessage : null)
  };
}
function getFormValidationResult(errorMessage, fields) {
  return {
    message: errorMessage != null ? errorMessage : "",
    hasError: !isEmpty(errorMessage != null ? errorMessage : null) || fields != null && Object.keys(fields).find((k) => fields[k].hasError) != null,
    fields: __spreadValues({}, fields)
  };
}
const SimpleTextTypes = [
  "text",
  "password",
  "date",
  "datetime-local",
  "email",
  "search",
  "url",
  "time",
  "month",
  "week",
  "tel"
];
const special_char_regex = /[!@#$%^&*()_+\-=\[\]{};':"\\|,.<>\/?]+/;
const MIME_IMAGES = ["image/png", "image/jpeg", "image/gif", "image/svg+xml", "image/webp", "image/bmp", "image/tiff"];
const MIME_VIDEO = ["video/mp4", "video/webv", "video/ogv"];
const MIME_AUDIO = ["audio/mpeg", "audio/ogg", "audio/x-m4a", "audio/3gpp"];
const MIME_PDF = ["application/pdf"];
const MIME_MS_EXCEL = ["vnd.ms-excel", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"];
const MIME_MS_WORD = ["application/msword", "application/vnd.openxmlformats-officedocument.wordprocessingml.document"];
function notEmpty(val, errorMessage = "Please, fill out this field.") {
  if (val == null)
    return errorMessage;
  if ((val == null ? void 0 : val.constructor) === FileList)
    return val.length == 0 ? errorMessage : "";
  if (isArray(val))
    return val.length == 0 ? errorMessage : "";
  if (typeof val === "string")
    return isEmpty(val) ? errorMessage : "";
  return "";
}
function validName(val, errorMessage = "Please, enter a valid name.") {
  if (/[<>/\\{}*#~`%]+$/.test(val))
    return errorMessage;
  return "";
}
function getMinLenValidator(length = 1) {
  return (val) => {
    if (isEmpty(val))
      return "Please, fill out this field.";
    else if (val.length < length)
      return `Should have at least ${length} characters.`;
    return "";
  };
}
function fileTypeValidator(fileMimeList, errorMessage = "Please, upload valid files only.") {
  return (val) => {
    const fileList = [...val];
    if (fileList.length == 0)
      return "";
    if (fileList.find((f) => fileMimeList.indexOf(f.type) == -1) != null)
      return errorMessage;
    return "";
  };
}
function imgTypeFile(errorMessage = "Please, upload images only") {
  return fileTypeValidator(MIME_IMAGES, errorMessage);
}
function getStrongPasswordValidator({minLength = 8, specialChars = true} = {}) {
  return (val) => {
    if (isEmpty(val))
      return "Please, fill out this field.";
    else if (val.length < minLength)
      return `Password should have at least ${minLength} characters.`;
    else if (val.toLowerCase() === val || val.toUpperCase() === val)
      return "Password should have at least 1 capital and 1 small letter.";
    else if (specialChars && !special_char_regex.test(val))
      return "Password should have at least one special character (*!@#$%^&*).";
    return "";
  };
}
function maxFileSize(val, errorMessage = "Please, upload a file no bigger than {0}", maxUploadFileSize = 1024 * 1024 * 10) {
  if ([...val].find((f) => f.size > maxUploadFileSize) != null)
    return errorMessage.replace("{0}", getFriendlyFileSize(maxUploadFileSize));
  return "";
}
async function parseApiValidationResult(apiResult) {
  let result = {
    hasError: false,
    message: "",
    fields: {}
  };
  if (apiResult && apiResult.code === "ValidationError" && apiResult.details) {
    let keys = Object.keys(apiResult.details);
    for (let i = 0; i < keys.length; i++) {
      let fieldId = keys[i];
      if (apiResult.details[fieldId].length) {
        result.hasError = true;
        fieldId = fieldId.slice(0, 1).toLowerCase() + fieldId.slice(1);
        result.fields[fieldId] = {
          hasError: true,
          message: "Please, check this input"
        };
      }
    }
  }
  return result;
}
function AbstractRating({max = 5, value = 0, inputAttrs = {}, field}) {
  return withState(value, (rating) => /* @__PURE__ */ h("div", {
    style: {display: "table"}
  }, /* @__PURE__ */ h("input", __spreadProps(__spreadValues({}, inputAttrs), {
    hidden: true,
    required: false,
    type: "number",
    value: rating.bind()
  })), /* @__PURE__ */ h("style", null, `
                .star {cursor: pointer; font-size: 1.5em; color: darkorange;}
                .star:hover {font-weight: bold}
            `), new Array(max).fill(0).map((v, i) => /* @__PURE__ */ h("span", {
    class: "star",
    onclick: () => !field.readonly && rating.set(i + 1)
  }, rating.get() > i ? "\u2605" : "\u2606"))));
}
function AbstractLabel({field, htmlAttrs = {}, validationResult}) {
  var _a, _b;
  validationResult != null ? validationResult : validationResult = getValidationResult();
  if (field.hideLabel)
    return "";
  if (validationResult.hasError && (field.type == "checkbox" || field.type == "radio")) {
    (_a = htmlAttrs.style) != null ? _a : htmlAttrs.style = {};
    htmlAttrs.style.color = "red";
  }
  if (field.type == "checkbox" || field.readonly) {
    (_b = htmlAttrs.style) != null ? _b : htmlAttrs.style = {};
    htmlAttrs.style.display = "inline-block";
  }
  const label = h("label", __spreadProps(__spreadValues({}, htmlAttrs), {for: field.id}));
  label.children.push(field.label);
  if (field.required)
    label.children.push(h("span", {style: {color: "red"}}, "*"));
  if (field.readonly)
    label.children.push(": ");
  return label;
}
function DefaultFormLayout({forObject, formConfig, validationResult, htmlAttrs = {}}) {
  var _a;
  let _formConfig = formConfig != null ? formConfig : createFormConfig(forObject);
  let result = h(_formConfig.excludeSubmitButton ? "div" : "form", __spreadValues(__spreadValues({}, getFormHtmlAttrs(_formConfig)), htmlAttrs));
  for (const fieldId in _formConfig.fieldsConfig) {
    if (!_formConfig.fieldsConfig.hasOwnProperty(fieldId))
      continue;
    let field = _formConfig.fieldsConfig[fieldId];
    let fieldSet = h("div", {});
    const fieldValidationResult = (_a = validationResult == null ? void 0 : validationResult.fields[fieldId]) != null ? _a : {hasError: false};
    let input = AbstractInput({field, value: forObject[fieldId], validationResult: validationResult == null ? void 0 : validationResult.fields[fieldId]});
    let label = AbstractLabel({field, validationResult: validationResult == null ? void 0 : validationResult.fields[fieldId]});
    if (field.type === "radio") {
      input = Object.keys(field.choices).map((k, i) => h("label", {}, input[i], " ", field.choices[k]));
    }
    if (field.type != "checkbox" || field.readonly)
      fieldSet.children.push(...toArray(label), " ", ...toArray(input));
    else
      fieldSet.children.push(...toArray(input), " ", ...toArray(label));
    if (fieldValidationResult.hasError)
      fieldSet.children.push(h("div", {style: {color: "red"}}, h("small", {}, fieldValidationResult.message)));
    result.children.push(fieldSet);
  }
  return result;
}
function AbstractInput({field, htmlAttrs, value, validationResult}) {
  var _a, _b, _c;
  validationResult != null ? validationResult : validationResult = getValidationResult();
  if (field.readonly) {
    if (field.type == "checkbox")
      return value ? "Yes" : "No";
    else if (field.type == "sourcecode") {
      return h("pre", {}, value);
    } else if (field.type == "markdown") {
      return h("div", {}, value);
    } else if (field.type == "select" || field.type == "radio") {
      return h("div", {}, field.choices[value]);
    }
    return `${value == null ? "" : value}`;
  }
  const eltAttrs = __spreadValues(__spreadValues({}, getFieldHtmlAttrs(field)), htmlAttrs);
  if (validationResult.hasError) {
    (_a = eltAttrs.style) != null ? _a : eltAttrs.style = {};
    eltAttrs.style.border = "1px solid red";
  }
  if (field.type == "textarea") {
    return h("textarea", __spreadValues({rows: 3}, eltAttrs), value || "");
  }
  if (field.type == "submit") {
    return h("input", __spreadValues({type: "submit", value: (_b = field.label) != null ? _b : "Submit"}, eltAttrs));
  }
  if (field.type == "checkbox") {
    return h("input", __spreadValues({type: "checkbox", checked: !!value}, eltAttrs));
  }
  if (field.type == "toggle") {
    return h("input", __spreadValues({type: "checkbox", checked: value != null, value: field.choices[0]}, eltAttrs));
  }
  if (field.type == "select") {
    return h("select", __spreadValues({}, eltAttrs), ...field.required ? [] : [h("option", {value: ""}, (_c = field.placeholder) != null ? _c : "")], ...Object.keys(field.choices).map((k) => h("option", {value: k, selected: k == `${value}` ? true : void 0}, field.choices[k])));
  }
  if (field.type == "radio") {
    return Object.keys(field.choices).map((k) => h("input", __spreadProps(__spreadValues({}, eltAttrs), {
      id: void 0,
      type: field.multiple ? "checkbox" : "radio",
      value: `${k}`,
      checked: (field.multiple ? value.indexOf(k) > -1 : k == value) ? true : void 0
    })));
  }
  if (field.type == "files")
    return h("input", __spreadProps(__spreadValues({}, eltAttrs), {type: "file", multiple: "multiple", files: value}));
  if (field.type == "file")
    return h("input", __spreadProps(__spreadValues({}, eltAttrs), {type: "file", files: value}));
  if (field.type == "number")
    return h("input", __spreadProps(__spreadValues({}, eltAttrs), {type: "number", value: `${value == null ? "" : value}`}));
  if (field.type == "name")
    return h("input", __spreadProps(__spreadValues({}, eltAttrs), {type: "text", value: `${value == null ? "" : value}`}));
  if (field.type == "range")
    return h("input", __spreadProps(__spreadValues({}, eltAttrs), {type: "range", value: `${value == null ? "" : value}`}));
  if (field.type == "money")
    return h("input", __spreadProps(__spreadValues({min: "0", step: "0.01"}, eltAttrs), {type: "number", value: `${value == null ? "" : value}`}));
  if (field.type == "rating")
    return AbstractRating({value, inputAttrs: eltAttrs, max: 5, field});
  if (field.type == "composite") {
    return h("div", {style: {margin: "5px", border: "1px solid gray", padding: "2px"}}, DefaultFormLayout({
      forObject: value,
      formConfig: createFormConfig(value, __spreadValues({excludeSubmitButton: true, excludeFormTag: true}, field.customOptions))
    }));
  }
  if (SimpleTextTypes.indexOf(field.type) > -1)
    return h("input", __spreadProps(__spreadValues({}, eltAttrs), {type: field.type, value: `${value == null ? "" : value}`}));
  console.warn(`Unsupported field type: '${field.type}' for field '${field.id}'.`);
  return "";
}
let customFieldRenderers = [];
function addCustomFieldRenderer(renderer) {
  const id = randomHash();
  customFieldRenderers.push(__spreadProps(__spreadValues({}, renderer), {id}));
  return id;
}
function findCustomRenderer(forType) {
  var _a;
  return (_a = customFieldRenderers.find((cfr) => typeof cfr.forType === "string" ? forType.toLowerCase() == cfr.forType.toLowerCase() : cfr.forType.indexOf(forType) > -1)) != null ? _a : null;
}
function createFormConfig(forObject, _config = {}) {
  _config != null ? _config : _config = {};
  let config = __spreadProps(__spreadValues({
    scale: 1,
    readonly: false,
    hideLabels: false,
    excludeSubmitButton: false,
    autoValidate: true,
    layout: DefaultFormLayout,
    noValidate: true
  }, _config), {
    fieldsConfig: __spreadValues(__spreadValues({}, Object.keys(forObject).reduce((a, fieldId) => __spreadProps(__spreadValues({}, a), {[fieldId]: null}), {})), _config.fieldsConfig),
    $$isComplete: true
  });
  if (!config.excludeSubmitButton) {
    config.fieldsConfig["$$submit"] = __spreadProps(__spreadValues({}, getDefaultFieldConfig("$$submit", "submit", config)), {
      hideLabel: true,
      label: "Submit",
      type: "submit"
    });
  }
  for (const fieldId in forObject) {
    if (!forObject.hasOwnProperty(fieldId))
      continue;
    config.fieldsConfig[fieldId] = createFieldConfig(fieldId, forObject[fieldId], config.fieldsConfig[fieldId], config);
  }
  return config;
}
function createFieldConfig(fieldId, fieldValue, fieldConfig, formConfig) {
  var _a, _b;
  let type = (_a = fieldConfig == null ? void 0 : fieldConfig.type) != null ? _a : guessType(fieldId, fieldValue);
  let result = __spreadValues(__spreadValues(__spreadValues({}, getDefaultFieldConfig(fieldId, type, formConfig)), guessConfig(fieldValue, type)), fieldConfig);
  if (isEmpty(result.label))
    result.label = humanize(fieldId);
  result.choices = (result == null ? void 0 : result.choices) == null ? {} : ((_b = result.choices) == null ? void 0 : _b.constructor) === Array ? result.choices.reduce((acc, b) => __spreadProps(__spreadValues({}, acc), {[b]: b}), {}) : result.choices;
  return result;
}
function getDefaultFieldConfig(fieldId, type, formConfig) {
  var _a, _b, _c;
  return {
    scale: (_a = formConfig == null ? void 0 : formConfig.scale) != null ? _a : 1,
    readonly: (_b = formConfig == null ? void 0 : formConfig.readonly) != null ? _b : false,
    hideLabel: (_c = formConfig == null ? void 0 : formConfig.hideLabels) != null ? _c : false,
    colSpan: 1,
    icon: "",
    helpText: "",
    validationResult: {
      message: "",
      hasError: false
    },
    id: fieldId,
    multiple: false,
    type,
    label: "",
    choices: []
  };
}
function guessType(fieldId, fieldValue) {
  let table = {
    "password$": "password",
    "email$": "email",
    "name$": "name",
    "quantity$|number$": "number",
    "^amount|amount$|^price|price$": "money",
    "^date|date$": "date",
    "^year|year$": "year",
    "^month|month$": "month",
    "^phone|phone$": "tel",
    "^language|language$": "language",
    "^rating|rating$": "rating"
  };
  const fieldIdLower = fieldId.toLowerCase();
  for (const reg in table) {
    if (RegExp(reg).test(fieldIdLower))
      return table[reg];
  }
  if (fieldValue == null)
    return "text";
  const jsType = typeof fieldValue;
  if (jsType === "boolean")
    return "checkbox";
  if (jsType === "string") {
    if (isDateTime(fieldValue))
      return "datetime-local";
    if (isDate(fieldValue))
      return "date";
    if (isTime(fieldValue))
      return "time";
    if (isYear(fieldValue))
      return "year";
    return "text";
  }
  if (jsType === "number")
    return "number";
  if (fieldValue.constructor === Array)
    return "radio";
  if (jsType === "object" && fieldValue.constructor === Object)
    return "composite";
  return "text";
}
function guessConfig(val, fieldType, fieldConfig) {
  let result = {};
  if (val != null && val.constructor === Array && fieldType == "radio") {
    result.choices = val.reduce((acc, el) => __spreadProps(__spreadValues({}, acc), {[el]: humanize(el)}), {});
    result.multiple = true;
  }
  if (fieldType == "password")
    result.required = true;
  return result;
}
function mergeValidationResults(results) {
  return results.reduce((acc, curr) => ({
    hasError: (curr == null ? void 0 : curr.hasError) || acc.hasError,
    message: !isEmpty(curr == null ? void 0 : curr.message) ? curr == null ? void 0 : curr.message : acc.message
  }), getValidationResult());
}
function mergeFieldsValidationResults(vrs) {
  return vrs.reduce((acc, i) => {
    return Array.from(new Set([...Object.keys(acc), ...Object.keys(i)])).reduce((accDict, iKey) => {
      var _a, _b, _c, _d, _e, _f;
      return __spreadProps(__spreadValues({}, accDict), {
        [iKey]: {
          hasError: ((_a = acc[iKey]) == null ? void 0 : _a.hasError) || ((_b = i[iKey]) == null ? void 0 : _b.hasError) || false,
          message: isEmpty((_c = acc[iKey]) == null ? void 0 : _c.message) ? (_e = (_d = i[iKey]) == null ? void 0 : _d.message) != null ? _e : "" : (_f = acc[iKey]) == null ? void 0 : _f.message
        }
      });
    }, {});
  }, {});
}
function mergeFormValidationResults(vrs) {
  return vrs.reduce((acc, i) => {
    return {
      hasError: acc.hasError || i.hasError,
      message: isEmpty(i.message) ? acc.message : i.message,
      fields: mergeFieldsValidationResults([acc.fields, i.fields])
    };
  }, getFormValidationResult());
}
function validateForm(forObject, _formConfig) {
  const m = validateFormInternal(forObject, _formConfig);
  if (m.isAsync)
    console.error("Trying to synchronously validate formConfig with async validators");
  const formVR = m.form;
  let result = {
    message: formVR.message,
    hasError: formVR.hasError,
    fields: mergeFieldsValidationResults([m.fields, m.form.fields])
  };
  if (!result.hasError) {
    let anyOfFieldsHasError = Object.keys(result.fields).reduce((p, k) => p || result.fields[k].hasError, false);
    result.hasError || (result.hasError = anyOfFieldsHasError);
  }
  return result;
}
async function validateFormAsync(forObject, _formConfig) {
  const m = validateFormInternal(forObject, _formConfig);
  const formVR = await m.form;
  let result = {
    message: formVR.message,
    hasError: formVR.hasError,
    fields: formVR.fields
  };
  for (const [k, valPromise] of Object.entries(m.fields)) {
    const vr = await valPromise;
    result.fields[k] = mergeValidationResults([vr, result.fields[k]]);
    result.hasError || (result.hasError = result.fields[k].hasError);
  }
  Object.keys(forObject).filter((k) => result.fields[k] == null).forEach((k) => result.fields[k] = getValidationResult());
  return result;
}
function validateFormInternal(forObject, _formConfig) {
  let formConfig = _formConfig != null ? _formConfig : createFormConfig(forObject, _formConfig);
  let fieldsConfig = formConfig.fieldsConfig;
  let result = {fields: {}, form: getFormValidationResult(), isAsync: false};
  for (const [id, value] of Object.entries(forObject)) {
    const config = fieldsConfig[id];
    result.fields[id] = getValidationResult();
    if (config == null)
      continue;
    let validators = toArray(config.validate);
    if (config.required && validators.indexOf(notEmpty) == -1)
      validators.push(notEmpty);
    else if (validators.length == 0)
      continue;
    let fieldVR = runValidator(validators, value);
    result.fields[id] = fieldVR;
    if (fieldVR.then !== void 0)
      result.isAsync = true;
  }
  result.form = formConfig.validate ? runFormValidator(formConfig.validate, forObject) : getFormValidationResult();
  if (result.form.then !== void 0)
    result.isAsync = true;
  return result;
}
function getFormVRFromValidation(validationResult) {
  if (validationResult == null)
    return getFormValidationResult();
  if (typeof validationResult === "string")
    return getFormValidationResult(validationResult);
  if (validationResult.then === void 0 && typeof validationResult === "object") {
    let fields = Object.keys(validationResult).reduce((acc, k) => __spreadProps(__spreadValues({}, acc), {[k]: getValidationResult(validationResult[k])}), {});
    return getFormValidationResult(null, fields);
  }
  return null;
}
function runFormValidator(validator, form) {
  let validators = toArray(validator);
  if (validators.length == 0)
    return getFormValidationResult();
  try {
    let asyncVRs = [];
    let syncVRs = getFormValidationResult();
    for (let i = 0; i < validators.length; i++) {
      const v = validators[i];
      if (v == null)
        continue;
      let vResult = v(form);
      if (vResult == null)
        continue;
      if (typeof vResult === "string") {
        if (!isEmpty(vResult))
          return getFormValidationResult(vResult);
        continue;
      }
      if (vResult.then === void 0 && typeof vResult === "object") {
        syncVRs = getFormValidationResult(syncVRs.message, mergeFieldsValidationResults([
          syncVRs.fields,
          Object.keys(vResult).reduce((acc, k) => __spreadProps(__spreadValues({}, acc), {[k]: getValidationResult(vResult[k])}), {})
        ]));
        continue;
      }
      asyncVRs.push(vResult.then((res) => {
        var _a;
        return (_a = getFormVRFromValidation(res)) != null ? _a : getFormValidationResult();
      }).catch((err) => {
        console.warn("Error while attempting to validate formConfig", err);
        return getFormValidationResult("Failed to validate this entry.");
      }));
    }
    if (syncVRs.hasError)
      return syncVRs;
    if (asyncVRs.length) {
      return Promise.all(asyncVRs).then((vrs) => mergeFormValidationResults(vrs));
    }
    return syncVRs;
  } catch (err) {
    return getFormValidationResult("Failed to validate the formConfig.");
  }
}
function runValidator(validator, value) {
  if (validator == null)
    return getValidationResult();
  try {
    const validators = toArray(validator);
    let asyncVRs = [];
    for (let i = 0; i < validators.length; i++) {
      const v = validators[i];
      if (v == null)
        continue;
      let vResult = v(value);
      if (vResult == null)
        continue;
      if (typeof vResult === "string") {
        if (!isEmpty(vResult))
          return getValidationResult(vResult);
        continue;
      }
      asyncVRs.push(vResult.then((errMsg) => getValidationResult(errMsg)).catch((err) => {
        console.warn("Error while attempting to validate", err);
        return getValidationResult("Failed to validate this entry.");
      }));
    }
    if (asyncVRs.length) {
      return Promise.all(asyncVRs).then((vrs) => mergeValidationResults(vrs));
    }
  } catch (ex) {
    return getValidationResult("Failed to validate this entry.");
  }
  return getValidationResult();
}
function getFieldConfigs(form) {
  return Object.keys(form.fieldsConfig).map((k) => form.fieldsConfig[k]);
}
function getFieldHtmlAttrs(field) {
  const src = __spreadProps(__spreadValues({
    name: field.id
  }, field), {
    icon: void 0,
    type: void 0,
    helpText: void 0,
    label: void 0,
    validationResult: void 0,
    customOptions: void 0,
    maxlength: void 0,
    multiple: void 0,
    group: void 0,
    choices: void 0,
    variation: void 0,
    hideLabel: void 0,
    scale: void 0,
    readonly: void 0,
    validate: void 0,
    onValidation: void 0
  });
  let result = {
    name: field.id
  };
  for (const [k, val] of Object.entries(src)) {
    if (val != null && val != "")
      result[k] = val;
  }
  return result;
}
function getFormHtmlAttrs(form) {
  const src = __spreadProps(__spreadValues({}, form), {
    fieldsConfig: void 0,
    validate: void 0,
    scale: void 0,
    hideLabels: void 0,
    excludeSubmitButton: void 0,
    readonly: void 0,
    validationResult: void 0,
    onValidation: void 0,
    excludeFormTag: void 0,
    $$isComplete: void 0,
    syncValues: void 0,
    autoValidate: void 0,
    layout: void 0
  });
  let result = {};
  for (const [key, val] of Object.entries(src)) {
    if (val != null && val != "")
      result[key] = val;
  }
  return result;
}
function renderForm(forObject, target, formConfig, validationResult) {
  validationResult = validationResult != null ? validationResult : getFormValidationResult();
  let _formConfig = formConfig != null && (formConfig == null ? void 0 : formConfig.$$isComplete) ? formConfig : createFormConfig(forObject, formConfig);
  let FormComponent = h(_formConfig.layout, {forObject, formConfig: _formConfig, validationResult});
  if (_formConfig.autoValidate && !_formConfig.excludeFormTag) {
    let onSubmit = _formConfig.onsubmit;
    _formConfig.onsubmit = void 0;
    let rendered = renderToDom(FormComponent, target);
    rendered.$$domElement.addEventListener("submit", (e) => {
      let state = getFormValue(_formConfig, e.target);
      let vr = validateForm(state, _formConfig);
      if (vr.hasError) {
        e.preventDefault();
        rendered.newProps((a) => __spreadProps(__spreadValues({}, a), {forObject: state, validationResult: vr}));
      } else {
        rendered.newProps((a) => __spreadProps(__spreadValues({}, a), {forObject: state, validationResult: vr}));
        if (onSubmit)
          onSubmit(e);
      }
    });
  } else {
    let domElt = toDomElement(FormComponent, target);
    target.append(domElt);
  }
}
function onFieldChangeReducer(form, event) {
  const input = event.target;
  if (isEmpty(input == null ? void 0 : input.name))
    return (v) => v;
  return (prevVal) => __spreadProps(__spreadValues({}, prevVal), {
    [input.name]: getChangedFieldValue(form, event.target)
  });
}
function getFieldIdFromElement(elt) {
  return elt.name;
}
function getChangedFieldValue(form, target) {
  if (target == null)
    return null;
  const fieldId = getFieldIdFromElement(target);
  const field = form.fieldsConfig[fieldId];
  return getFieldValue(fieldId, field, [target]);
}
function getFieldValue(fieldId, field, fieldElements) {
  if (field == null)
    return null;
  let val = "";
  fieldElements != null ? fieldElements : fieldElements = [...document.getElementsByName(fieldId)];
  if (fieldElements.length == 0)
    return null;
  let fieldElement = fieldElements[0];
  let input = fieldElement;
  if (field.type == "radio") {
    let radios = fieldElements;
    val = field.multiple ? radios.filter((r) => r.checked).map((r) => r.value) : input.checked ? input.value : "";
  } else if (field.type == "checkbox")
    val = input.checked;
  else if (field.type == "file" || field.type == "files")
    val = input.files;
  else if (field.type == "number")
    val = parseFloat(input.value);
  else
    val = input.value;
  return val;
}
function getFormValue(form, formElement) {
  let result = {};
  for (const [fieldId, field] of Object.entries(form.fieldsConfig)) {
    result[fieldId] = getFieldValue(fieldId, field);
  }
  return result;
}
export {AbstractInput, AbstractLabel, AbstractRating, DefaultFormLayout, MIME_AUDIO, MIME_IMAGES, MIME_MS_EXCEL, MIME_MS_WORD, MIME_PDF, MIME_VIDEO, SimpleTextTypes, addCustomFieldRenderer, createFieldConfig, createFormConfig, fileTypeValidator, findCustomRenderer, getChangedFieldValue, getDefaultFieldConfig, getFieldConfigs, getFieldHtmlAttrs, getFieldIdFromElement, getFieldValue, getFormHtmlAttrs, getFormValidationResult, getFormValue, getMinLenValidator, getStrongPasswordValidator, getValidationResult, guessConfig, guessType, imgTypeFile, maxFileSize, notEmpty, onFieldChangeReducer, parseApiValidationResult, renderForm, runFormValidator, runValidator, validName, validateForm, validateFormAsync};
